@@ -119,12 +119,6 @@ impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> Generic<T, C> {
            self.cursor,
            self.buffer.as_ref().map(|buffer| buffer.len()));
 
-        // See if there is an error from the last invocation.
-        if let Some(e) = self.error.take() {
-            t!("Returning stashed error: {}", e);
-            return Err(e);
-        }
-
         if let Some(ref buffer) = self.buffer {
             // We have a buffer.  Make sure `cursor` is sane.
             assert!(self.cursor <= buffer.len());
@@ -139,8 +133,9 @@ impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> Generic<T, C> {
             // The caller wants more data than we have readily
             // available.  Read some more.
 
-            let capacity : usize = amount
-                + cmp::max(default_buf_size(), 2 * self.preferred_chunk_size);
+            let capacity : usize = amount.saturating_add(
+                default_buf_size().max(
+                    self.preferred_chunk_size.saturating_mul(2)));
 
             let mut buffer_new = self.unused_buffer.take()
                 .map(|mut v| {
@@ -159,6 +154,12 @@ impl<T: io::Read + Send + Sync, C: fmt::Debug + Sync + Send> Generic<T, C> {
                     break;
                 }
 
+                // See if there is an error from the last invocation.
+                if let Some(e) = &self.error {
+                    t!("We have a stashed error, don't poll again: {}", e);
+                    break;
+                }
+
                 match self.reader.read(&mut buffer_new
                                        [amount_buffered + amount_read..]) {
                     Ok(read) => {
@@ -349,8 +350,7 @@ mod test {
 
         // Same test, but as a slice.
         {
-            let mut data : &[u8] = include_bytes!("buffered-reader-test.txt");
-            let mut bio = Generic::new(&mut data, None);
+            let mut bio = Generic::new(crate::BUFFERED_READER_TEST_DATA, None);
 
             buffered_reader_test_data_check(&mut bio);
         }
@@ -404,4 +404,29 @@ mod test {
             }
         }
     }
+
+    /// Tests that we can request some data using data_hard even if a
+    /// previous request for more data failed.
+    #[test]
+    fn data_hard_after_failure() -> io::Result<()> {
+        /// Returns one byte once, then errors.
+        #[derive(Default)]
+        struct BuggySource(bool);
+        impl io::Read for BuggySource {
+            fn read(&mut self, _: &mut [u8]) -> io::Result<usize> {
+                if self.0 {
+                    Err(io::Error::new(io::ErrorKind::Other, "oops"))
+                } else {
+                    self.0 = true;
+                    Ok(1)
+                }
+            }
+        }
+
+        let mut br = Generic::new(BuggySource::default(), None);
+        assert!(br.data(2).is_ok()); // Ok...
+        assert_eq!(br.data(2).unwrap().len(), 1); // ... but short.
+        assert!(br.data_hard(1).is_ok()); // Should be fine then.
+        Ok(())
+    }
 }
