@@ -759,13 +759,14 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
             assert!(t[0] <= t[1]);
         }
 
+        let buf_size = default_buf_size();
         let mut total = 0;
         let position = 'outer: loop {
             let len = {
                 // Try self.buffer.  Only if it is empty, use
                 // self.data.
                 let buffer = if self.buffer().is_empty() {
-                    self.data(default_buf_size())?
+                    self.data(buf_size)?
                 } else {
                     self.buffer()
                 };
@@ -848,12 +849,13 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
     /// discards it.  A consequence of this is that an error may occur
     /// after we have consumed some of the data.
     fn drop_eof(&mut self) -> Result<bool, std::io::Error> {
+        let buf_size = default_buf_size();
         let mut at_least_one_byte = false;
         loop {
-            let n = self.data(default_buf_size())?.len();
+            let n = self.data(buf_size)?.len();
             at_least_one_byte |= n > 0;
             self.consume(n);
-            if n < default_buf_size() {
+            if n < buf_size {
                 // EOF.
                 break;
             }
@@ -862,6 +864,37 @@ pub trait BufferedReader<C> : io::Read + fmt::Debug + fmt::Display + Send + Sync
         Ok(at_least_one_byte)
     }
 
+    /// Copies data to the given writer returning the copied amount.
+    ///
+    /// This is like using [`std::io::copy`], but more efficient as it
+    /// avoids an extra copy, and it will try to copy all the data the
+    /// reader has already buffered.
+    ///
+    /// On success, returns the amount of data (in bytes) that has
+    /// been copied.
+    ///
+    /// Note: this function reads and copies the data a chunk at a
+    /// time.  A consequence of this is that an error may occur after
+    /// we have consumed some of the data.
+    fn copy(&mut self, sink: &mut dyn io::Write) -> io::Result<u64> {
+        let buf_size = default_buf_size();
+        let mut total = 0;
+        loop {
+            let data = self.data(buf_size)?;
+            sink.write_all(data)?;
+
+            let n = data.len();
+            total += n as u64;
+            self.consume(n);
+            if n < buf_size {
+                // EOF.
+                break;
+            }
+        }
+
+        Ok(total)
+    }
+
     /// A helpful debugging aid to pretty print a Buffered Reader stack.
     ///
     /// Uses the Buffered Readers' `fmt::Display` implementations.
@@ -1083,14 +1116,17 @@ fn buffered_reader_test_data_check<'a, T: BufferedReader<C> + 'a, C: fmt::Debug
     }
 }
 
+#[cfg(test)]
+const BUFFERED_READER_TEST_DATA: &[u8] =
+    include_bytes!("buffered-reader-test.txt");
+
 #[cfg(test)]
 mod test {
     use super::*;
 
     #[test]
     fn buffered_reader_eof_test() {
-        let data : &[u8] = include_bytes!("buffered-reader-test.txt");
-
+        let data = BUFFERED_READER_TEST_DATA;
         // Make sure data_eof works.
         {
             let mut bio = Memory::new(data);
@@ -1168,8 +1204,7 @@ mod test {
 
     #[test]
     fn buffered_reader_read_test() {
-        let data : &[u8] = include_bytes!("buffered-reader-test.txt");
-
+        let data = BUFFERED_READER_TEST_DATA;
         {
             let bio = Memory::new(data);
             buffered_reader_read_test_aux (bio, data);
@@ -1228,4 +1263,24 @@ mod test {
         // Matches EOF.
         assert!(reader.drop_through(b"def", true).unwrap().0.is_none());
     }
+
+    #[test]
+    fn copy() -> io::Result<()> {
+        // The memory reader has all the data buffered, copying it
+        // will issue a single write.
+        let mut bio = Memory::new(BUFFERED_READER_TEST_DATA);
+        let mut sink = Vec::new();
+        let amount = bio.copy(&mut sink)?;
+        assert_eq!(amount, 50_000);
+        assert_eq!(&sink[..], BUFFERED_READER_TEST_DATA);
+
+        // The generic reader uses buffers of the given chunk size,
+        // copying it will issue multiple writes.
+        let mut bio = Generic::new(BUFFERED_READER_TEST_DATA, Some(64));
+        let mut sink = Vec::new();
+        let amount = bio.copy(&mut sink)?;
+        assert_eq!(amount, 50_000);
+        assert_eq!(&sink[..], BUFFERED_READER_TEST_DATA);
+        Ok(())
+    }
 }
